layout(location = 0) out vec3 outColor0;

const float ambientOcclusionLevel = 1.0; // [0.0 0.1 0.2 0.3 0.4 0.5 0.6 0.7 0.8 0.9 1.0]

uniform sampler2D colortex0;

uniform sampler2D depthtex0;

uniform float far;
uniform float near;

in vec2 texCoord;

#ifndef VISUALIZE_CULLING
	const float ambient = AMBIENT_LIGHT * 0.1;
#else
	const vec3 ambient = vec3(1000.0, 0.0, 0.0);
#endif

float depth = texture(depthtex0, texCoord).r;

float linearDepth = 2.0 * near * far / (far + near - (2.0 * depth - 1.0) * (far - near));

#ifdef AUTOMATIC_EXPOSURE
	const float eyeBrightnessHalflife = 2.5; // [0.0 0.5 1.0 1.5 2.0 2.5 3.0 3.5 4.0 4.5 5.0 5.5 6.0 6.5 7.0 7.5 8.0 8.5 9.0 9.5 10.0]

	uniform int worldTime;

	uniform ivec2 eyeBrightnessSmooth;

	float getAutomaticExposure() {
		float torchLight = eyeBrightnessSmooth.s;
		torchLight /= 30.0;

		float skyLight = eyeBrightnessSmooth.t;
		skyLight /= 240.0;
		skyLight += 0.5 - distance(worldTime, 6000.0) / 12000.0;

		return max(1.0, 3.0 - mix(torchLight, 2.0, clamp(skyLight, 0.0, 1.0)));
	}
#endif

#if HDR_OVEREXPOSED != 100 || HDR_UNDEREXPOSED != 100
	vec3 getHDR(in vec3 color) {
		const float overExposed = HDR_OVEREXPOSED * 0.01;
		const float underExposed = HDR_UNDEREXPOSED * 0.01;

		vec3 colorOverExposed = color * overExposed;
		vec3 colorUnderExposed = color * underExposed;

		return mix(colorUnderExposed, colorOverExposed, color);
	}
#endif

#if TONEMAP == 3 // uncharted2Tonemap
	vec3 uncharted2Math(in vec3 color) {
		return ((color * (0.15 * color + 0.05) + 0.004) / (color * (0.15 * color + 0.5) + 0.06)) - 0.02 / 0.3;
	}
#endif

vec3 gamma(in vec3 color) {
	return pow(color, vec3(1.0 / GAMMA));
}

float luminance(in vec3 color) {
	return dot(color, vec3(0.2125, 0.7154, 0.0721));
}

vec3 tonemap(in vec3 color) {
	#if TONEMAP == 0 // reinhardJodieTonemap
		color = pow(color, vec3(TONEMAP_CURVE));

		float luminance = luminance(color) + 1.0;

		vec3 tv = color / (1.0 + color);

		color = mix(color / luminance, tv, tv);

		color = pow(color, vec3(1.0 / TONEMAP_CURVE));

		return gamma(color);
	#elif TONEMAP == 1 // burgessHDRTonemap
		vec3 overExposed = color * 1.2;
		vec3 underExposed = color * 0.6;

		color = mix(underExposed, overExposed, color);

		color = max(vec3(0.0), color - 0.004);

		color = (color * (6.2 * color + 0.05)) / (color * (6.2 * color + 2.3) + 0.06);

		return gamma(color);
	#elif TONEMAP == 2 // ACESTonemap
		const float a = 2.51;
		const float b = 0.03;
		const float c = 2.43;
		const float d = 0.59;
		const float e = 0.14;

		color = clamp((color * (a * color + b)) / (color * (c * color + d) + e), 0.0, 1.0);

		return gamma(color);
	#elif TONEMAP == 3 // uncharted2Tonemap
		const vec3 whiteScale = vec3(1.0) / uncharted2Math(vec3(11.2));

		color = uncharted2Math(color * 2.0);

		color *= whiteScale;

		return gamma(color);
	#elif TONEMAP == 4 // lottesTonemap
		const vec3 hdrMax = vec3(8.0);
		const vec3 midIn = vec3(0.18);
		const vec3 midOut = vec3(0.267);

		const vec3 a = vec3(1.6);
		const vec3 d = a * vec3(0.977);

		const vec3 b = (-pow(midIn, a) + pow(hdrMax, a) * midOut) / ((pow(hdrMax, d) - pow(midIn, d)) * midOut);
		const vec3 c = (pow(hdrMax, d) * pow(midIn, a) - pow(hdrMax, a) * pow(midIn, d) * midOut) / ((pow(hdrMax, d) - pow(midIn, d)) * midOut);

		color = pow(color, a) / (pow(color, d) * b + c);

		return gamma(color);
	#elif TONEMAP == 5 // filmicTonemap
		color = max(vec3(0.0), color - 0.004);

		color = (color * (6.2 * color + 0.5)) / (color * (6.2 * color + 1.7) + 0.06);

		return color;
	#else
		return gamma(color);
	#endif
}

#if FOG_STRENGTH
	in float fogStrength;

	uniform vec3 fogColor;
	uniform vec3 skyColor;
#endif

void main() {
	vec3 color = pow(texture(colortex0, texCoord).rgb, vec3(2.2));

	if (depth < 1.0) {
		#if AMBIENT_LIGHT != 10
			color *= AMBIENT_LIGHT * 0.1;
		#endif

		#if FOG_STRENGTH
			float fogDistance = clamp(pow(pow(distance(texCoord.x, 0.5), 2.0) * 100.0 + linearDepth, 4.0) * fogStrength * 0.25, 0.0, 1.0);
			color = mix(color, mix(fogColor, skyColor, fogDistance), fogDistance);
		#endif
	}

	/* works with blindness but not sun :(
	#if FOG_STRENGTH
		color = mix(color, fogColor, clamp(pow(pow(distance(texCoord.x, 0.5), 2.0) * 100.0 + linearDepth, 4.0) * fogStrength * 0.25, 0.0, 1.0));
	#endif
	*/

	#if RED != 100 || GREEN != 100 || BLUE != 100
		color *= vec3(RED, GREEN, BLUE) * 0.01;
	#endif

	color = vec3(1.0) - exp(-color * EXPOSURE);

	#ifdef AUTOMATIC_EXPOSURE
		color *= getAutomaticExposure();
	#endif

	#if HDR_OVEREXPOSED != 100 || HDR_UNDEREXPOSED != 100
		color = getHDR(color);
	#endif

	color = tonemap(color);

	outColor0 = color * (0.2 * color + 0.9);
}